<?php
/**
 * Theme setup class.
 *
 * Set up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support post thumbnails.
 *
 */

class Weeland_Setup {

  // Theme version
  protected $version;
  
  // Theme unique id
  protected $theme_slug;
  
  public function __construct() {
    $this->version    = '1.2.2';
    $this->theme_slug = 'weeland';
  }
  
  /**
   * Setup theme. Add actions, filters, features etc.
   *
   * @since Weeland 1.0
   */   
  public function setup() {
    global $content_width;

    /**
     * Set up the content width value based on the theme's design.
     */
    if ( ! isset( $content_width ) ) {
      $content_width = 750;
    }
  
    /*
     * Make theme available for translation.
     *
     * Translations can be added to the /languages/ directory.
     */
    load_theme_textdomain( 'weeland', get_template_directory() . '/languages' );
  
    // Add RSS feed links to <head> for posts and comments.
    add_theme_support( 'automatic-feed-links' );

    // Enable support for Post Thumbnails, and declare two sizes.
    add_theme_support( 'post-thumbnails' );
    set_post_thumbnail_size( 750, 9999, false );
    add_image_size( 'weeland-grid-thumbnail', 550, 9999, false );
    add_image_size( 'weeland-medium-thumbnail', 500, 300, true );
    
    // Register navigation menus.
    register_nav_menus( array(
      'primary' => esc_html__( 'Primary Navigation', 'weeland' )
    ) );
    
    /*
     * Switch default core markup for search form, comment form, and comments
     * to output valid HTML5.
     */
    add_theme_support( 'html5', array(
      'search-form', 'comment-form', 'comment-list', 'gallery', 'caption'
    ) );
    
    // Add document title tag to HTML <head>.
    add_theme_support( 'title-tag' );
    
    // Setup the WordPress core custom header image.  
    add_theme_support( 'custom-header', array( 
      'width'         => 500,
      'height'        => 1100,
      'flex-height'   => true,
      'flex-width'    => true,
      'default-image' => get_template_directory_uri() . '/assets/images/default-sidebar-image.jpg'
    ) );

    // Add theme support for Custom Logo.
    add_theme_support( 'custom-logo', array(
      'flex-width' => true,
    ) );
    
    // Add public scripts and styles
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ), 10 );
    add_action( 'wp_enqueue_scripts', array( $this, 'post_nav_background' ), 10 );
    
    // Widgets init
    add_action( 'widgets_init', array( $this, 'widgets_init' ) );
    
    // Modify Read More link
    add_filter( 'the_content_more_link', array( $this, 'modify_read_more_link' ) );
    
    // Modify search form
    add_filter( 'get_search_form', array( $this, 'search_form' ) );
    
    // Set new excerpt more text
    add_filter('excerpt_more', array( $this, 'custom_excerpt_more' ));
    
    // This theme uses its own gallery styles.
    add_filter( 'use_default_gallery_style', '__return_false' );
    
    // Add classes to body tag and posts.
    add_filter( 'body_class', array( $this, 'add_body_classes' ) );
    
    // Remove custom styles and texts transients when this updated
    add_action( 'customize_save_after', array( $this, 'reset_custom_styles_cache' ) );
    add_action( 'customize_save_after', array( $this, 'reset_custom_texts_cache' ) );

    // Modification archives title
    add_filter( 'get_the_archive_title', array( $this, 'archive_title' ), 10, 1 );

    /*
     * This theme styles the visual editor to resemble the theme style,
     * specifically font, colors, icons, and column width.
     */
    add_editor_style( array( 'assets/css/editor-style.css', $this->fonts_url() ) );

    // Comment form fields
    add_filter( 'comment_form_default_fields', array( $this, 'comment_form_default_fields' ), 10, 1 );
  }
  
  /**
   * Register and enqueue theme scripts.
   *
   * @since Weeland 1.0
   */  
  public function enqueue_scripts() {
    // Enqueue scripts
    if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
      wp_enqueue_script( 'comment-reply' );
    }

    wp_register_script( 'owl-carousel', get_template_directory_uri() . '/assets/js/owl.carousel.min.js', array( 'jquery' ), '2.2.1', true );

    wp_enqueue_script( 'fitvids', get_template_directory_uri() . '/assets/js/jquery.fitvids.min.js', array( 'jquery' ), '1.1', true );
    wp_enqueue_script( 'weeland-functions', get_template_directory_uri() . '/assets/js/functions.js', array( 'jquery' ), $this->version, true );     
  }
  
  /**
   * Register and enqueue theme styles.
   *
   * @since Weeland 1.0
   */    
  public function enqueue_styles() {
    // Add fonts, used in the main stylesheet.
    wp_enqueue_style( 'weeland-fonts', $this->fonts_url(), array(), $this->version );

    wp_register_style( 'owl-carousel', get_template_directory_uri() . '/assets/css/owl.carousel.min.css', array(), '2.2.1' );
    
    // Font Awesome icons font style
    wp_register_style( 'font-awesome', get_template_directory_uri() . '/assets/css/font-awesome.min.css', array(), '4.7.0' );  
    
    // Load our main stylesheet.
    wp_enqueue_style( 'weeland-style', get_stylesheet_uri(), array( 'font-awesome' ), $this->version );
    
    // Add customizer css.
    wp_add_inline_style( 'weeland-style', $this->inline_style() );
  }
  
  /**
   * Register Google fonts for Weeland.
   *
   * @since Weeland 1.0
   *
   * @return string Google fonts URL for the theme.
   */  
  public function fonts_url() {
    $fonts_url = '';
    $fonts     = array();
    $subsets   = 'latin,latin-ext';

    /* translators: If there are characters in your language that are not supported by Lora, translate this to 'off'. Do not translate into your own language. */
    if ( 'off' !== esc_attr_x( 'on', 'Lora font: on or off', 'weeland' ) ) {
      $fonts[] = 'Lora:400,400italic,700,700italic';
    }

    /* translators: If there are characters in your language that are not supported by Oswald, translate this to 'off'. Do not translate into your own language. */
    if ( 'off' !== esc_attr_x( 'on', 'Oswald font: on or off', 'weeland' ) ) {
      $fonts[] = 'Oswald:400,600';
    }

    /* translators: To add an additional character subset specific to your language, translate this to 'greek', 'cyrillic', 'devanagari' or 'vietnamese'. Do not translate into your own language. */
    $subset = esc_attr_x( 'no-subset', 'Add new subset (greek, cyrillic, devanagari, vietnamese)', 'weeland' );

    if ( 'cyrillic' == $subset ) {
      $subsets .= ',cyrillic,cyrillic-ext';
    } elseif ( 'greek' == $subset ) {
      $subsets .= ',greek,greek-ext';
    } elseif ( 'devanagari' == $subset ) {
      $subsets .= ',devanagari';
    } elseif ( 'vietnamese' == $subset ) {
      $subsets .= ',vietnamese';
    }

    if ( $fonts ) {
      $fonts_url = esc_url( add_query_arg( array(
        'family' => urlencode( implode( '|', $fonts ) ),
        'subset' => urlencode( $subsets ),
      ), '//fonts.googleapis.com/css' ) );
    }

    return $fonts_url;
  }
  
  /**
   * Inline style. Used in wp_add_inline_style.
   *
   * @since Weeland 1.0
   */  
  public function inline_style() {
    if( is_customize_preview() )
      return $this->get_inline_style();
      
    $inline_style = get_transient( 'weeland_inline_style' );
    
    if( $inline_style === false ) {
      $inline_style = $this->get_inline_style();
      set_transient( 'weeland_inline_style', $inline_style );
    }

    if( $this->get_sidebar_image_url() ) {
      $inline_style .= sprintf( '
        .sidebar {
          background-image: url( %s )
        }',
        esc_url( $this->get_sidebar_image_url() )
      );
    }

    if( get_theme_mod( 'mobile_sidebar_image' ) ) {
      $single_sidebar_image_id = false;

      if( is_single() || is_page() ) {
        global $post;

        $single_sidebar_image_id = absint( get_post_meta( $post->ID, 'weeland_sidebar_image_id', true ) );
      }

      $mobile_image = wp_get_attachment_image_src( absint( get_theme_mod( 'mobile_sidebar_image' ) ), 'large' );

      $mobile_image_url = $mobile_image[0];

      if( $mobile_image_url && ! $single_sidebar_image_id ) {
        $inline_style .= sprintf( '
          @media (max-width: 991px) {
            .sidebar {
              background-image: url( %s )
            }
          }',
          esc_url( $mobile_image_url )
        );
      }
    }

    if ( is_singular() && comments_open() ) {
      $inline_style .= sprintf( '
        .bypostauthor .fn:after {
          content: "%s";
        }',
        esc_html__( 'Author', 'weeland' )
      );
    }
    
    return $inline_style;
  }
  
  /**
   * Get inline style CSS.
   *
   * @since Weeland 1.0
   *
   * @return string Css with custom styles.
   */  
  public function get_inline_style() {
    $css = '';

    $sidebar_image_blackout = get_theme_mod( 'sidebar_image_blackout' );
    
    if( is_numeric( $sidebar_image_blackout ) && $sidebar_image_blackout >=0 && $sidebar_image_blackout <= 1 ){
      $css .= sprintf( '
        .has-sidebar-image .sidebar:before{
          opacity: %s;
        }',
        $sidebar_image_blackout
      );
    }
    
    $logo_size = absint( get_theme_mod( 'logo_size' ) );
    
    if( $logo_size ){
      $css .= sprintf( '
        .custom-logo-link img{
          max-width: %dpx;
        }',
        $logo_size
      );
    }

    $link_color = $this->sanitize_hex_color( get_theme_mod( 'link_color' ) );
    
    if( $link_color ) {
      $css .= sprintf( '
        .page-content a,
        .taxonomy-description a,
        .post-meta a,
        .entry-content a,
        .author-description a,
        .logged-in-as a,
        .comment-content a,
        .comment-edit-link,
        .comment-navigation a,
        .textwidget a,
        .site-info a {
          color: %s;
        }',
        $link_color
      );
    }
    
    $link_color_hover = $this->sanitize_hex_color( get_theme_mod( 'link_color_hover' ) );
    
    if( $link_color_hover ) {
      $css .= sprintf( '
        .page-content a:hover,
        .taxonomy-description a:hover,
        .post-meta a:hover,
        .entry-content a:hover,
        .author-description a:hover,
        .logged-in-as a:hover,
        .comment-content a:hover,
        .comment-edit-link:hover,
        .comment-navigation a:hover,
        .textwidget a:hover,
        .site-info a:hover {
          color: %s;
        }',
        $link_color_hover
      );
    }
    
    $button_background_color = $this->sanitize_hex_color( get_theme_mod( 'button_background_color' ) );
    
    if( $button_background_color ) {
      $css .= sprintf( '
        button,
        input[type="button"],
        input[type="reset"],
        input[type="submit"] {
          background-color: %s;
        }',
        $button_background_color
      );
    }
    
    $button_background_color_hover = $this->sanitize_hex_color( get_theme_mod( 'button_background_color_hover' ) );
    
    if( $button_background_color_hover ) {
      $css .= sprintf( '
        button:hover,
        input[type="button"]:hover,
        input[type="reset"]:hover,
        input[type="submit"]:hover {
          background-color: %s;
        }',
        $button_background_color_hover
      );
    }
    
    $button_text_color = $this->sanitize_hex_color( get_theme_mod( 'button_text_color' ) );
    
    if( $button_text_color ) {
      $css .= sprintf( '
        button,
        input[type="button"],
        input[type="reset"],
        input[type="submit"] {
          color: %s;
        }',
        $button_text_color
      );
    }

    $category_underline_color = $this->sanitize_hex_color( get_theme_mod( 'category_underline_color' ) );
    
    if( $category_underline_color ) {
      $css .= sprintf( '
        .cat-links a {
          border-bottom-color: %s;
        }',
        $category_underline_color
      );
    }

    $pagination_current_page_color = $this->sanitize_hex_color( get_theme_mod( 'pagination_current_page_color' ) );
    
    if( $pagination_current_page_color ) {
      $css .= sprintf( '
        .page-numbers:hover, 
        .page-numbers.current {
          background: %s;
        }',
        $pagination_current_page_color
      );
    }

    if( $this->get_sidebar_image_url() && is_customize_preview() ) {
      $css .= sprintf( '
        .sidebar {
          background-image: url( %s )
        }',
        esc_url( $this->get_sidebar_image_url() )
      );
    }

    if( get_theme_mod( 'mobile_sidebar_image' ) && is_customize_preview() ) {
      $single_sidebar_image_id = false;

      if( is_single() || is_page() ) {
        global $post;

        $single_sidebar_image_id = absint( get_post_meta( $post->ID, 'weeland_sidebar_image_id', true ) );
      }

      $mobile_image = wp_get_attachment_image_src( absint( get_theme_mod( 'mobile_sidebar_image' ) ), 'large' );

      $mobile_image_url = $mobile_image[0];

      if( $mobile_image_url && ! $single_sidebar_image_id ) {
        $css .= sprintf( '
          @media (max-width: 991px) {
            .sidebar {
              background-image: url( %s )
            }
          }',
          esc_url( $mobile_image_url )
        );
      }
    }
    
    return $css;
  }
  
  /**
   * Sanitizes a hex color.
   *
   * @since Weeland 1.0
   */  
  public function sanitize_hex_color( $color ) {
    if ( '' === $color )
      return '';
      
    // 3 or 6 hex digits, or the empty string.
    if ( preg_match('|^#([A-Fa-f0-9]{3}){1,2}$|', $color ) )
      return $color; 
  }

  /**
   * Get the sidebar image URL.
   *
   * @since Weeland 1.0
   */  
  public function get_sidebar_image_url() {
    if( is_single() || is_page() ) {
      global $post;

      $sidebar_image_id = absint( get_post_meta( $post->ID, 'weeland_sidebar_image_id', true ) );

      if( $sidebar_image_id ) {
        $sidebar_image_src = wp_get_attachment_image_src( $sidebar_image_id, 'weeland-sidebar-image' );

        if( is_array( $sidebar_image_src ) )
          return $sidebar_image_src[0];
      }
    }

    if( has_header_image() ) {
      $sidebar_image_url = get_header_image();

      return $sidebar_image_url;
    }
  }
  
  /**
   * Add featured image as background image to post navigation elements.
   *
   * @since Weeland 1.0
   *
   * @see wp_add_inline_style()
   */  
  function post_nav_background() {
    if ( ! is_single() ) {
      return;
    }

    $previous = ( is_attachment() ) ? get_post( get_post()->post_parent ) : get_adjacent_post( false, '', true );
    $next     = get_adjacent_post( false, '', false );
    $css      = '';

    if ( is_attachment() && 'attachment' == $previous->post_type ) {
      return;
    }

    if ( $previous &&  has_post_thumbnail( $previous->ID ) ) {
      $prevthumb = wp_get_attachment_image_src( get_post_thumbnail_id( $previous->ID ), 'post-thumbnail' );
      $css .= '
        .post-navigation .nav-previous { 
          background-image: url(' . esc_url( $prevthumb[0] ) . '); 
        }
        .post-navigation .nav-previous .post-title, 
        .post-navigation .nav-previous a:hover .post-title { 
          color: #fff; 
        }
        .post-navigation .nav-previous .nav-meta {
          color: #fff;
          border-color: rgba(255,255,255,0.3);
        }
        .post-navigation .nav-previous .nav-meta:hover {
          border-color: rgba(255,255,255,0.35);
        }
        .post-navigation .nav-previous:before { 
          background-color: rgba(0, 0, 0, 0.4); 
        }
        .post-navigation .nav-previous:hover:before { 
          background-color: rgba(0, 0, 0, 0.8); 
        }
      ';
    }

    if ( $next && has_post_thumbnail( $next->ID ) ) {
      $nextthumb = wp_get_attachment_image_src( get_post_thumbnail_id( $next->ID ), 'post-thumbnail' );
      $css .= '
        .post-navigation .nav-next { 
          background-image: url(' . esc_url( $nextthumb[0] ) . '); 
        }
        .post-navigation .nav-next .post-title, 
        .post-navigation .nav-next a:hover .post-title { 
          color: #fff; 
        }
        .post-navigation .nav-next .nav-meta {
          color: #fff;
          border-color: rgba(255,255,255,0.3);
        }
        .post-navigation .nav-next .nav-meta:hover {
          border-color: rgba(255,255,255,0.35);
        }
        .post-navigation .nav-next:before { 
          background-color: rgba(0, 0, 0, 0.4); 
        }
        .post-navigation .nav-next:hover:before { 
          background-color: rgba(0, 0, 0, 0.8); 
        }
      ';
    }

    wp_add_inline_style( 'weeland-style', $css );
  }
  
  /**
   * Reset custom styles cache.
   *
   * @since Weeland 1.0
   */
  function reset_custom_styles_cache() {
    delete_transient( 'weeland_inline_style' );
  }
  
  /**
   * Reset custom text cache.
   *
   * @since Weeland 1.0
   */
  function reset_custom_texts_cache() {
    delete_transient( 'weeland_tagline' );
    delete_transient( 'weeland_copyright' );
  }

  /**
   * Register widget area.
   *
   * @since Weeland 1.0
   *
   * @link https://codex.wordpress.org/Function_Reference/register_sidebar
   */
  public function widgets_init() {
    require get_template_directory() . '/inc/widgets.php';
    register_widget( 'Weeland_Instagram_Widget' );
    
    // Register widgets
    register_sidebar( array(
      'name'          => esc_html__( 'Footer Widgets &mdash; Column 1', 'weeland' ),
      'id'            => 'widget-area-1',
      'description'   => esc_html__( 'Add widgets here to appear in footer.', 'weeland' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );

    register_sidebar( array(
      'name'          => esc_html__( 'Footer Widgets &mdash; Column 2', 'weeland' ),
      'id'            => 'widget-area-2',
      'description'   => esc_html__( 'Add widgets here to appear in footer.', 'weeland' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );
  }
  
  /**
   * More link modification.
   *
   * @since Weeland 1.0
   *
   * @return string Modified more link.
   */  
  public function modify_read_more_link() {
    return sprintf( '<p class="entry-more"><a class="more-link" href="%s">%s</a></p>', esc_url( get_permalink() ), esc_html__( 'Continue Reading &rarr;', 'weeland' ) );
  }
  
  /**
   * Search form modification.
   *
   * @since Weeland 1.0
   *
   * @return string HTML with search form.
   */   
  public function search_form( $form ) {
  
    $form = '
      <form method="get" class="search-form" action="' . esc_url( home_url( '/' ) ) . '">
        <div class="search-wrap">
          <label>
            <input type="search" class="search-field" placeholder="' . esc_attr_x( 'Search&hellip;', 'placeholder', 'weeland' ) . '" value="' . get_search_query() . '" name="s" title="' . esc_attr_x( 'Search for:', 'title tag', 'weeland' ) . '" />
          </label>
          <button type="submit" class="search-submit"><i class="fa fa-search"></i></button>
        </div>
			</form>';
			
    return $form;
  }
  
  /**
   * Modification default excerpt more.
   *
   * @since Weeland 1.0
   *
   * @return string Excerpt more text.
   */   
  public function custom_excerpt_more( $more ) {
    return '...';
  }
  
   /**
   * Add classes to body tag.
   *
   * @since Weeland 1.0
   *
   * @return array Body classes array.
   */    
  public function add_body_classes( $classes ) {
    if( has_header_image() ) {
      $classes[] = 'has-sidebar-image';
    } else if( is_single() || is_page() ) {
      
      global $post;

      $sidebar_image_id = get_post_meta( $post->ID, 'weeland_sidebar_image_id', true );
      $sidebar_image_src = wp_get_attachment_image_src( $sidebar_image_id, 'weeland-sidebar-image' );

      if( is_array( $sidebar_image_src ) )
        $classes[] = 'has-sidebar-image';
    }

    return $classes;
  }

  /**
   * Changing default archive titles.
   *
   * @since Weeland 1.0
   */    
  public function archive_title( $title ) {
    if ( is_category() ) {
      return single_cat_title( '', false );
    }

    if ( is_author() ) {
      return get_the_author();
    }

    return $title;
  }

  /**
   * Modification of the default fields of the comment form..
   *
   * @since Weeland 1.2.1
   */  
  public function comment_form_default_fields( $fields ) {
    $commenter = wp_get_current_commenter();

    $req      = get_option( 'require_name_email' );
    $aria_req = ( $req ? " aria-required='true'" : '' );
    $html_req = ( $req ? " required='required'" : '' );
    $html5    = 'html5';

    $fields['author'] = '
      <p class="comment-form-author">
        <input id="author" name="author" type="text" placeholder="' . esc_attr__( 'Name', 'weeland' ) . ( $req ? ' *' : '' ) . '" value="' . esc_attr( $commenter['comment_author'] ) . '" size="30"' . $aria_req . $html_req . ' />
      </p>';

    $fields['email'] = '
      <p class="comment-form-email">
        <input id="email" name="email" ' . ( $html5 ? 'type="email"' : 'type="text"' ) . ' placeholder="' . esc_attr__( 'Email', 'weeland' ) . ( $req ? ' *' : '' ) . '" value="' . esc_attr(  $commenter['comment_author_email'] ) . '" size="30" aria-describedby="email-notes"' . $aria_req . $html_req  . ' />
      </p>';

    $fields['url'] = '
      <p class="comment-form-url">
        <input id="url" name="url" ' . ( $html5 ? 'type="url"' : 'type="text"' ) . ' placeholder="' . esc_attr__( 'Website', 'weeland' ) . '" value="' . esc_attr( $commenter['comment_author_url'] ) . '" size="30" />
      </p>';

    return $fields;
  }
}

$weeland_setup = new Weeland_Setup();

add_action( 'after_setup_theme', array( $weeland_setup, 'setup' ), 10 );

/**
 * Customizer additions.
 *
 * @since Weeland 1.0
 */
require get_template_directory() . '/inc/customizer.php';

/**
 * Custom template tags for this theme.
 *
 * @since Weeland 1.0
 */
require get_template_directory() . '/inc/template-tags.php';

/**
 * Theme Metaboxes
 *
 * @since Weeland 1.0
 */
require get_template_directory() . '/inc/class-weeland-metaboxes-setup.php';

/**
 * Custom Galleries
 *
 * @since Weeland 1.2
 */
require get_template_directory() . '/inc/class-weeland-custom-galleries.php';