<?php
/**
 * Adding additional types of galleries.
 *
 * @since Weeland 1.2
 */
 
class Weeland_Custom_Galleries {

	protected $gallery_types;
	protected $default_gallery_type;

	public function setup() {
		$this->gallery_types = array(
			'default'  => esc_html__( 'Thumbnail Grid', 'weeland' ),
			'carousel' => esc_html__( 'Carousel', 'weeland' ),
			'masonry'  => esc_html__( 'Masonry', 'weeland' )
		);

		$this->default_gallery_type = 'default';

		if ( ( class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'tiled-gallery' ) ) || class_exists( 'themePacific_Jetpack_Tiled_Gallery' ) ) {
			add_filter( 'jetpack_gallery_types', array( $this, 'add_gallery_types' ), 10 );
		} else {
			add_action( 'wp_enqueue_media', array( $this, 'wp_enqueue_media' ) );
			add_action( 'print_media_templates', array( $this, 'print_media_templates' ) );
		}

		add_filter( 'post_gallery', array( $this, 'gallery_shortcode' ), 10, 3 );
	}

	public function add_gallery_types( $types = array() ) {
		$types = array_merge( $types, $this->gallery_types );
		return $types;
	}

	public function wp_enqueue_media() {
		wp_enqueue_script( 'weeland-custom-gallery-settings', get_template_directory_uri() . '/assets/js/admin/gallery-settings.js', array( 'media-views' ) );
	}

	function print_media_templates() {
		?>
		<script type="text/html" id="tmpl-weeland-gallery-settings">
			<label class="setting">
				<span><?php esc_html_e( 'Type', 'weeland' ); ?></span>
				<select class="type" name="type" data-setting="type">
					<?php foreach ( $this->gallery_types as $gallery_type_id => $gallery_type_name ) : ?>
						<option value="<?php echo esc_attr( $gallery_type_id ); ?>" <?php selected( $gallery_type_id, $this->default_gallery_type ); ?>><?php echo esc_html( $gallery_type_name ); ?></option>
					<?php endforeach; ?>
				</select>
			</label>
		</script>
		<?php
	}

	function gallery_shortcode( $output = '', $attr, $instance ) {
		if ( ! array_key_exists( 'type', $attr ) ) 
			return;

		$gallery_type = $attr['type'];

		if( in_array( $gallery_type, array( 'carousel', 'masonry' ) ) ) {
			$post = get_post();

			static $instance = 0;
			$instance++;

			if ( ! empty( $attr['ids'] ) ) {
				// 'ids' is explicitly ordered, unless you specify otherwise.
				if ( empty( $attr['orderby'] ) ) {
					$attr['orderby'] = 'post__in';
				}
				$attr['include'] = $attr['ids'];
			}

			$html5 = current_theme_supports( 'html5', 'gallery' );
			$atts = shortcode_atts( array(
				'order'      => 'ASC',
				'orderby'    => 'menu_order ID',
				'id'         => $post ? $post->ID : 0,
				'itemtag'    => $html5 ? 'figure'     : 'dl',
				'icontag'    => $html5 ? 'div'        : 'dt',
				'captiontag' => $html5 ? 'figcaption' : 'dd',
				'columns'    => 3,
				'size'       => 'thumbnail',
				'include'    => '',
				'exclude'    => '',
				'link'       => ''
			), $attr, 'gallery' );

			$id = intval( $atts['id'] );

			if ( ! empty( $atts['include'] ) ) {
				$_attachments = get_posts( array( 'include' => $atts['include'], 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $atts['order'], 'orderby' => $atts['orderby'] ) );

				$attachments = array();
				foreach ( $_attachments as $key => $val ) {
					$attachments[$val->ID] = $_attachments[$key];
				}
			} elseif ( ! empty( $atts['exclude'] ) ) {
				$attachments = get_children( array( 'post_parent' => $id, 'exclude' => $atts['exclude'], 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $atts['order'], 'orderby' => $atts['orderby'] ) );
			} else {
				$attachments = get_children( array( 'post_parent' => $id, 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $atts['order'], 'orderby' => $atts['orderby'] ) );
			}

			if ( empty( $attachments ) ) {
				return '';
			}

			if ( is_feed() ) {
				$output = "\n";
				foreach ( $attachments as $att_id => $attachment ) {
					$output .= wp_get_attachment_link( $att_id, $atts['size'], true ) . "\n";
				}
				return $output;
			}

			$itemtag = tag_escape( $atts['itemtag'] );
			$captiontag = tag_escape( $atts['captiontag'] );
			$icontag = tag_escape( $atts['icontag'] );
			$valid_tags = wp_kses_allowed_html( 'post' );
			if ( ! isset( $valid_tags[ $itemtag ] ) ) {
				$itemtag = 'dl';
			}
			if ( ! isset( $valid_tags[ $captiontag ] ) ) {
				$captiontag = 'dd';
			}
			if ( ! isset( $valid_tags[ $icontag ] ) ) {
				$icontag = 'dt';
			}

			$columns = intval( $atts['columns'] );

			$selector = "gallery-{$instance}";

			$size_class = sanitize_html_class( $atts['size'] );
			$output = '';

			switch ( $gallery_type ) {
				case 'masonry': 
					wp_enqueue_script( 'jquery-masonry' );
					break;
				case 'carousel':
					wp_enqueue_script( 'owl-carousel' );
					wp_enqueue_style( 'owl-carousel' );
					wp_enqueue_script( 'imagesloaded' );
					break;
			}

			$gallery_div = "<div id='$selector' class='gallery gallery-columns-{$columns} weeland-custom-gallery'>";
			$gallery_type_wrap = '';

			switch ( $gallery_type ) {
				case 'masonry': 
					$gallery_type_wrap .= "<div class='masonry-container masonry-gallery' data-columns='{$columns}'>";
					break;
				case 'carousel':
					$gallery_type_wrap .= "<div class='owl-carousel carousel-gallery' data-columns='{$columns}'>";
					break;
			}

			$output .= $gallery_div . $gallery_type_wrap;

			$i = 0;
			foreach ( $attachments as $id => $attachment ) {

				$attr = ( trim( $attachment->post_excerpt ) ) ? array( 'aria-describedby' => "$selector-$id" ) : '';
				if ( ! empty( $atts['link'] ) && 'file' === $atts['link'] ) {
					$image_output = wp_get_attachment_link( $id, $atts['size'], false, false, false, $attr );
				} elseif ( ! empty( $atts['link'] ) && 'none' === $atts['link'] ) {
					$image_output = wp_get_attachment_image( $id, $atts['size'], false, $attr );
				} else {
					$image_output = wp_get_attachment_link( $id, $atts['size'], true, false, false, $attr );
				}
				$image_meta  = wp_get_attachment_metadata( $id );

				$orientation = '';
				if ( isset( $image_meta['height'], $image_meta['width'] ) ) {
					$orientation = ( $image_meta['height'] > $image_meta['width'] ) ? 'portrait' : 'landscape';
				}
				$output .= "<{$itemtag} class='gallery-item'>";
				$output .= "
					<{$icontag} class='gallery-icon {$orientation}'>
						$image_output
					</{$icontag}>";
				if ( $captiontag && trim($attachment->post_excerpt) ) {
					$output .= "
						<{$captiontag} class='wp-caption-text gallery-caption' id='$selector-$id'>
						" . wptexturize($attachment->post_excerpt) . "
						</{$captiontag}>";
				}
				$output .= "</{$itemtag}>";
				if ( ! $html5 && $columns > 0 && ++$i % $columns == 0 ) {
					$output .= '<br style="clear: both" />';
				}
			}

			if ( ! $html5 && $columns > 0 && $i % $columns !== 0 ) {
				$output .= "
					<br style='clear: both' />";
			}

			$output .= "
					</div>
				</div>\n";
		}

		return $output;
	}
}

$weeland_custom_galleries = new Weeland_Custom_Galleries();

add_action( 'after_setup_theme', array( $weeland_custom_galleries, 'setup' ), 15 );
?>