<?php
/**
 * Post options metaboxes.
 *
 * @since Weeland 1.0
 */
 
class Weeland_Metaboxes_Setup {

  protected $custom_sidebar_image_screens = array( 'post', 'page', 'portfolio' );

  protected $default_page_options = array(
    'display_title' => true,
    'display_share_buttons' => true
  );

  public function setup() {
    add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
		add_action( 'add_meta_boxes',        array( $this, 'add_meta_boxes' ) );
		add_action( 'save_post',             array( $this, 'save_custom_sidebar_image_id' ) );
    add_action( 'save_post_page',        array( $this, 'save_page_options_meta' ) );
  }
  
  /**
	 * Enqueue meta boxes scripts and styles.
	 */	
	public function admin_enqueue_scripts() {     
    wp_enqueue_script( 'weeland-custom-sidebar-image-js',  get_template_directory_uri() . '/assets/js/admin/custom-sidebar-image.js', array( 'jquery' ), '1.0', true );

    wp_localize_script( 
      'weeland-custom-sidebar-image-js', 
      'weelandVars', 
      array( 
        'mediaFrameTitle'  => esc_html__( 'Select Image', 'weeland' ),
        'mediaFrameButton' => esc_html__( 'Choose Image', 'weeland' )
      )
    );  
	}
  
  // Add meta boxes
  public function add_meta_boxes() {
    foreach( $this->custom_sidebar_image_screens as $screen ) {
      add_meta_box( 'weeland-sidebar-image', esc_html__( 'Sidebar Image', 'weeland' ), array( $this, 'custom_sidebar_image_view' ),  $screen, 'side', 'low' );
    }

    add_meta_box( 'weeland-page-options', esc_html__( 'Page Options', 'weeland' ), array( $this, 'page_options_view' ),  array( 'page' ), 'side' );
  }
  
  public function custom_sidebar_image_view( $post ) {
    // Get WordPress' media upload URL
    $upload_link = get_upload_iframe_src( 'image', $post->ID );

    // See if there's a media id already saved as post meta
    $sidebar_image_id = get_post_meta( $post->ID, 'weeland_sidebar_image_id', true );

    // Get the image src
    $sidebar_image_src = wp_get_attachment_image_src( $sidebar_image_id, 'weeland-sidebar-image' );

    // For convenience, see if the array is valid
    $has_sidebar_image = is_array( $sidebar_image_src );
    ?>

    <div class="custom-sidebar-image-container">
    <?php if ( $has_sidebar_image ) : ?>
      <img src="<?php echo esc_url( $sidebar_image_src[0] ); ?>" style="max-width:100%;" />
    <?php endif; ?>
    </div>

    <p class="hide-if-no-js">
      <a class="upload-custom-sidebar-image <?php if ( $has_sidebar_image  ) { echo 'hidden'; } ?>" href="<?php echo esc_url( $upload_link ) ?>">
        <?php esc_html_e( 'Set sidebar image', 'weeland' ); ?>
      </a>
      <a class="delete-custom-sidebar-image <?php if ( ! $has_sidebar_image  ) { echo 'hidden'; } ?>" href="#">
          <?php esc_html_e( 'Remove sidebar image', 'weeland' ); ?>
      </a>
    </p>

    <?php wp_nonce_field( 'save_custom_sidebar_image_meta', 'custom_sidebar_image_metabox_nonce' ); ?>

    <input class="custom-sidebar-image-id" name="custom-sidebar-image-id" type="hidden" value="<?php echo absint( $sidebar_image_id ); ?>" />
  <?php
  }

  public function page_options_view( $post ) {
    $page_options = get_post_meta( $post->ID, 'weeland_page_options', true );
    $page_options = wp_parse_args( $page_options, $this->default_page_options );
  ?>
    <p>
      <label class="selectit"><input type="checkbox" id="display-title" name="weeland_display_title" <?php checked( $page_options['display_title'], true ); ?>><?php esc_html_e( 'Display title', 'weeland' ); ?></label>
    </p>
    <p>
      <label class="selectit"><input type="checkbox" id="display-post-thumbnail" name="weeland_display_share_buttons" <?php checked( $page_options['display_share_buttons'], true ); ?>><?php esc_html_e( 'Display share buttons', 'weeland' ); ?></label>
    </p>
  <?php
    wp_nonce_field( 'weeland_save_page_options_meta', 'weeland_page_options_meta_box_nonce' );
  }

  public function save_custom_sidebar_image_id( $post_id ) {
    // Checks save status
    $is_autosave = wp_is_post_autosave( $post_id );
    $is_revision = wp_is_post_revision( $post_id );
    $is_valid_nonce = ( isset( $_POST[ 'custom_sidebar_image_metabox_nonce' ] ) && wp_verify_nonce( $_POST[ 'custom_sidebar_image_metabox_nonce' ], 'save_custom_sidebar_image_meta' ) ) ? true : false;
 
    // Exits script depending on save status
    if ( $is_autosave || $is_revision || !$is_valid_nonce ) {
        return;
    }

    if( isset( $_POST[ 'custom-sidebar-image-id' ] ) ) {
      update_post_meta( $post_id, 'weeland_sidebar_image_id', absint( $_POST[ 'custom-sidebar-image-id' ] ) );
    }
  }

  // Save page options meta
  public function save_page_options_meta( $post_id ) {
    $is_autosave = wp_is_post_autosave( $post_id );
    $is_revision = wp_is_post_revision( $post_id );
    $is_valid_nonce = ( isset( $_POST[ 'weeland_page_options_meta_box_nonce' ] ) && wp_verify_nonce( $_POST[ 'weeland_page_options_meta_box_nonce' ], 'weeland_save_page_options_meta' ) ) ? true : false;
 
    if ( $is_autosave || $is_revision || !$is_valid_nonce ) {
        return;
    }

    $display_title = ( isset( $_POST[ 'weeland_display_title' ] ) ) ? true : false;
    $display_share_buttons = ( isset( $_POST[ 'weeland_display_share_buttons' ] ) ) ? true : false;

    update_post_meta( $post_id, 'weeland_page_options', compact( 'display_title', 'display_share_buttons' ) );
  }
}

$weeland_metaboxes = new Weeland_Metaboxes_Setup();

add_action( 'after_setup_theme', array( $weeland_metaboxes, 'setup' ), 16 );
?>