<?php
/**
 * Custom template tags for Weeland
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @since Weeland 1.0
 */

if ( ! function_exists( '_wp_render_title_tag' ) ) :
  /**
   * Add wp_title on head if theme not supported "title-tag" feature.
   *
   * @since Weeland 1.0
   */
  function weeland_render_title() {
  ?>
    <title><?php wp_title( '|', true, 'right' ); ?></title>
  <?php
  }

  add_action( 'wp_head', 'weeland_render_title' );
endif;

if ( ! function_exists( 'weeland_site_identity' ) ) :
  /**
   * Prints HTML with blog title and description.
   *
   * @since Weeland 1.0
   */
  function weeland_site_identity() {
    if( get_theme_mod( 'custom_logo' ) ) {
      weeland_custom_logo();
    }

    if( get_theme_mod( 'display_site_title', 1 ) ) {
      printf( '<%1$s class="site-title"><a href="%2$s" rel="home">%3$s</a></%1$s>', 
        is_front_page() && is_home() ? 'h1' : 'p', 
        esc_url( home_url( '/' ) ), 
        esc_html( get_bloginfo( 'name' ) ) 
      );
    }

    weeland_tagline( '<div class="tagline">', '</div>' );
  }
endif;

if ( ! function_exists( 'weeland_custom_logo' ) ) :
  /**
   * Displays the optional custom logo.
   *
   * Does nothing if the custom logo is not available.
   *
   * @since Weeland 1.0
   */
  function weeland_custom_logo() {
    if ( function_exists( 'the_custom_logo' ) ) {
      the_custom_logo();
    }
  }
endif;

if ( ! function_exists( 'weeland_tagline' ) ) :
  /**
   * Prints HTML with custom header text.
   *
   * @since Weeland 1.0
   */
  function weeland_tagline( $before = '', $after = '' ) {
    $header_text = get_transient( 'weeland_tagline' );
    
    if( $header_text === false || is_customize_preview() ) {
      $header_text = get_theme_mod( 'tagline' );
      
      if( $header_text ) {
        $header_text = wp_kses( $header_text, array(
            'a' => array(
              'href' => array(),
              'title' => array()
            ),
            'p'      => array(),
            'b'      => array(),
            'strong' => array(),
            'em'     => array(),
            'i'      => array(),
            'br'     => array(),
            'span'   => array(),
            'img' => array(
              'src' => array(),
              'alt' => array(),
              'title' => array()
            )
          )
        );
      }
      set_transient( 'weeland_tagline', $header_text );
    }
    
    if( $header_text || is_customize_preview() )
      printf( '%s%s%s', $before, $header_text, $after );
  }
endif;

if ( ! function_exists( 'weeland_site_copyright' ) ) :
  /**
   * Prints HTML with custom or default copyright text.
   *
   * @since Weeland 1.0
   */
  function weeland_site_copyright( $before = '', $after = '' ) {
    $default_copyright = sprintf( '&copy; %1$d <a href="%2$s">%3$s</a>', date('Y'), esc_url( home_url( '/' ) ), esc_html( get_bloginfo( 'name' ) ) );
    
    $copyright = get_transient( 'weeland_copyright' );
    
    if( $copyright === false || is_customize_preview() ) {
      $copyright = get_theme_mod( 'copyright' );
      
      if( $copyright ) {
        $copyright = wp_kses( $copyright, array(
            'a' => array(
              'href' => array(),
              'title' => array()
            ),
            'p'      => array(),
            'b'      => array(),
            'strong' => array(),
            'em'     => array(),
            'i'      => array(),
            'br'     => array(),
            'span'   => array(),
            'img' => array(
              'src' => array(),
              'alt' => array(),
              'title' => array()
            )
          )
        );
      }
      set_transient( 'weeland_copyright', $copyright );
    }
    
    printf( '%s%s%s', $before, $copyright ? $copyright : $default_copyright, $after );
  }
endif;

if ( ! function_exists( 'weeland_social_profiles' ) ) :
  /**
   * Prints HTML with social profiles buttons.
   *
   * @since Weeland 1.0
   */
  function weeland_social_profiles( $before = '', $after = '' ) {
    $social_profiles = get_theme_mod( 'social' );
    
    if( empty( $social_profiles ) )
      return;
    
    $output = '';
    
    if( is_array( $social_profiles ) ) {
      foreach( $social_profiles as $network_id => $url ) {
        if( $url )
          $output .= sprintf( 
            '<li><a href="%1$s" class="%2$s" target="_blank"><i class="fa fa-%2$s"></i></a></li>', 
            ( $network_id == 'email' ) ? 'mailto:' . esc_attr( $url ) : esc_url( $url ), 
            esc_attr( $network_id ) 
         );
      }
    }
    
    if( $output )
      printf( '%s<ul class="social-profiles">%s</ul>%s', $before, $output, $after );
  }
endif;

if ( ! function_exists( 'weeland_post_thumbnail' ) ) :
  /**
   * Display an optional post thumbnail.
   *
   * Wraps the post thumbnail in an anchor element on index views, or a div
   * element when on single views.
   *
   * @since Weeland 1.0
   */
  function weeland_post_thumbnail( $size = 'post-thumbnail' ) {
    if ( post_password_required() || is_attachment() || ! has_post_thumbnail()  ) {
      return;
    }

    if ( is_singular() ) :
    ?>

    <div class="post-thumbnail">
      <?php the_post_thumbnail( $size ); ?>
    </div><!-- .post-thumbnail -->

    <?php else : ?>

    <a class="post-thumbnail" href="<?php the_permalink(); ?>" aria-hidden="true">
      <?php the_post_thumbnail( $size, array( 'alt' => get_the_title() ) ); ?>
    </a>

    <?php endif; // End is_singular()
  }
endif;

if ( ! function_exists( 'weeland_entry_meta' ) ) :
  /**
   * Prints HTML with meta information for the categories, post date.
   *
   * @since Weeland 1.0
   */
  function weeland_entry_meta( $before = '', $after = '', $display_date = true, $display_author = true ) {
    $output = '';
    $post_type = get_post_type();

    if ( in_array( $post_type, array( 'post', 'attachment' ) ) && $display_date ) {
      $time_string = sprintf( '<time class="entry-date published" datetime="%1$s">%2$s</time>',
        esc_attr( get_the_date( 'c' ) ),
        get_the_date()
      );

      $output .= sprintf( '<span class="posted-on post-meta">%s</span>', sprintf( esc_html__( 'Posted on %s', 'weeland' ), $time_string ) );
    }

    if ( 'post' == $post_type && $display_author ) {
      $author_string = sprintf( '<a class="url fn" href="%s">%s</a>', esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ), get_the_author() );
      $author_string = sprintf( esc_html_x( 'by %s', 'Post author', 'weeland' ), $author_string );
    
      $output .= sprintf( '<span class="byline author vcard post-meta">%s</span>', $author_string );
    }

    if( $output )
      printf( '%s%s%s', $before, $output, $after );
  }
endif;

if ( ! function_exists( 'weeland_category_list' ) ) :
  /**
   * Prints HTML with post categories list.
   *
   * @since Weeland 1.0
   */
  function weeland_category_list() {
    global $post;

    if( ! $post )
      return;

    $post_type = $post->post_type;
    
    if ( 'post' == $post_type ) {
      $categories_list = get_the_category_list( ' ' );

      if( $categories_list ) {
        printf( '<div class="cat-links">%s</div>', $categories_list );
      } 
    }

    return;
  }
endif;

if ( ! function_exists( 'weeland_entry_footer' ) ) :
  /**
   * Prints HTML with post tags and share buttons in post footer
   *
   * @since Weeland 1.0
   */
  function weeland_entry_footer( $before = '', $after = '' ) {
    $output = '';
    $post = get_post();
    
    if ( is_single() ) {
      if( 'post' == $post->post_type && get_theme_mod( 'display_tags_list', 1 ) ) {
        $output .= get_the_tag_list( '<div class="tags-list">', '', '</div>' );
      }
    }

    if( ( 'post' == $post->post_type && get_theme_mod( 'display_share_buttons', 1 ) ) || is_attachment() ) {
      $output .= sprintf( '<div class="entry-share">%s</div>', weeland_get_share_buttons() );
    }

    if( is_page() ) {
      $page_options = get_post_meta( $post->ID, 'weeland_page_options', true );
      
      if( get_theme_mod( 'display_page_share_buttons', 1 ) && ( isset( $page_options['display_share_buttons'] ) ? $page_options['display_share_buttons'] : true ) ) {
        $output .= sprintf( '<div class="entry-share">%s</div>', weeland_get_share_buttons() );
      }
    }
    
    if( $output )
      printf( '%s%s%s', $before, $output, $after );
  }
endif;

/**
 * Get HTML with post share buttons.
 *
 * @since Weeland 1.0
 */
function weeland_get_share_buttons() {
  $post = get_post();

  if( !$post )
    return;

  $thumbnail_src = '';
  $thumbnail = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'post-thumbnail' );
  
  if( $thumbnail )
    $thumbnail_src = urlencode( esc_url( $thumbnail['0'] ) );

  $post_permalink = urlencode( esc_url( get_permalink( $post->ID ) ) );
  $title          = urlencode( strip_tags( get_the_title() ) );
  $excerpt        = urlencode( strip_tags( get_the_excerpt() ) );
  
  $buttons = array(
    'facebook' => array(
      'url'    => 'http://www.facebook.com/sharer/sharer.php?u=' . $post_permalink,
      'title'  => esc_html__( 'Share on Facebook', 'weeland' ),
      'icon'   => '<i class="fa fa-facebook"></i>',
      'show'   => 1
     ),
    'twitter' => array(
      'url'    => 'https://twitter.com/intent/tweet?text=' . $title . '&amp;url=' . $post_permalink,
      'title'  => esc_html__( 'Tweet It', 'weeland' ),
      'icon'   => '<i class="fa fa-twitter"></i>',
      'show'   => 1
     ),
    'google' => array(
      'url'    => 'https://plus.google.com/share?url=' . $post_permalink,
      'title'  => esc_html__( 'Share on Google Plus', 'weeland' ),
      'icon'   => '<i class="fa fa-googleplus"></i>',
      'show'   => 1
     ),
    'pinterest' => array(
      'url'    => 'https://www.pinterest.com/pin/create/button/?description=' . $title . '&amp;media=' . $thumbnail_src . '&amp;url=' . $post_permalink,
      'title'  => esc_html__( 'Pin It', 'weeland' ),
      'icon'   => '<i class="fa fa-pinterest"></i>',
      'show'   => 1
     )
  );
  
  $output = '';

  foreach( $buttons as $button_id => $button_data ) {
    if( get_theme_mod( 'display_' . $button_id . '_share_button', $button_data['show'] ) ) {
      $output .= sprintf( '<a href="%s" title="%s" class="weeland-share-button weeland-%s-share-button" rel="nofollow">%s</a>', esc_url( $button_data['url'] ), esc_attr( $button_data['title'] ), esc_attr( $button_id ), $button_data['icon'] );
    }
  } 
  
  if( $output )
    return sprintf( '<div class="share-buttons">%s</div>', $output );
}

if ( ! function_exists( 'weeland_share_buttons' ) ) :
  /**
   * Prints HTML with post share buttons.
   *
   * @since Weeland 1.0
   */
  function weeland_share_buttons( $before = '', $after = '' ) {
    printf( '%s%s%s', $before, weeland_get_share_buttons(), $after );
  }
endif;

if ( ! function_exists( 'weeland_display_page_title' ) ) :
  /**
   * Checking if the page title is displayed or not.
   *
   * @since Weeland 1.0
   */
  function weeland_display_page_title() {
    global $post;

    if( !$post )
      return false;

    $page_options = get_post_meta( $post->ID, 'weeland_page_options', true );
    $display_page_title = isset( $page_options['display_title'] ) ? $page_options['display_title'] : true;

    return (bool) $display_page_title;
  }
endif;

if ( ! function_exists( 'weeland_related_posts' ) ) :
  /**
   * Related posts for current post.
   *
   * @since Weeland 1.0
   */
  function weeland_related_posts( $before = '', $after = '' ) {
    global $post;
    
    if( !$post )
      return;

    if( $post->post_type !== 'post' )
      return;
      
    $query_args = array(
      'post__not_in'        => array( $post->ID ),
      'posts_per_page'      => 3,
      'ignore_sticky_posts' => true
    );
    
    $related_by = get_theme_mod( 'related_by', 'none' );
    
    switch ( $related_by ) {
      case 'category' :
        $categories = get_the_category( $post->ID );
        if ( $categories ) {
          $category_ids = array();
          foreach( $categories as $category ) 
            $category_ids[] = $category->term_id;
            
          $query_args['category__in'] = $category_ids;
        }
        break;
      
      case 'tags' :
        $tags = wp_get_post_tags( $post->ID );
        if ( $tags ) {
          $tag_ids = array();
          foreach( $tags as $tag ) 
            $tag_ids[] = $tag->term_id;
            
          $query_args['tag__in'] = $tag_ids;
        }
        break;
      case 'none' :
        $query_args['orderby'] = 'rand';
        break;
    }
    
    $related_posts = new WP_Query( $query_args );
    $output = '';
    
    if( $related_posts->have_posts() ) {
      printf( '%s<div class="related-posts">', $before );
      
      while( $related_posts->have_posts() ) : $related_posts->the_post(); ?>
      <div class="related-post<?php if( has_post_thumbnail() ) : ?> has-post-thumbnail<?php endif; ?>">
      
        <?php if( has_post_thumbnail() ) : ?>
        <a href="<?php the_permalink()?>" class="post-thumbnail" rel="bookmark" title="<?php the_title(); ?>">
          <?php the_post_thumbnail( 'weeland-medium-thumbnail' ); ?>
        </a>
        <?php endif; ?>
        
        <div class="related-post-content">
          <a href="<?php the_permalink(); ?>" rel="bookmark" title="<?php the_title(); ?>"><?php the_title(); ?></a>
          <?php printf( '<div class="post-meta">%s</div>', get_the_date() ); ?>
        </div>
      </div>
    <?php 
      endwhile;
      
      printf( '</div>%s', $after );
    }

    wp_reset_postdata(); 
  }
endif;
?>